import numpy as np
import xarray as xr
import sasktran as sk
from scipy.interpolate import interp1d
import re
from skamf.NO2_VMR.climatology_G5NR import load_ds, HSP_to_altitude_state


def default_hsp_atmosphere():
    """
    Creates the default HSP (Hybrid Sigma Pressure) atmosphere definition that was used by the first iterations of
    the AMF software.

    Consists of NO2, O3, and temperature profiles at each of the 72 layer centres.

    The AMF code will take these inputs as well as a surface pressure and convert them to altitude-dependent
    profiles for sasktran. The conversion will use the barometric formula assuming ideal gas and lapse rate = 0
    (constant temperature within each layer). It will also produce an air density profiles using the ideal gas law.
    """
    # hard coded settings
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    # take a ozone altitude profile and convert it to HSP assuming surface pressure of 101.325 kPa (atmospheric
    # sea level) at an altitude of 0
    ds = load_ds()

    atmo = HSP_to_altitude_state(101325.0, 0.0, ds['T'])

    altitude = atmo.altitude.values
    density = atmo.density.values
    o3_clim = sk.Labow()
    o3_prof_cm3 = o3_clim.get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)
    o3_prof_vmr = o3_prof_cm3 / density

    # identify variables defining the HSP atmosphere by the attribute 'HSP_atmosphere'
    data_vars = {
        'a_no2_vmr': ('a_sigma_levels', ds['TRCNO2'].values, {'HSP_atmosphere': 'NO2_VMR'}),
        'a_o3_vmr': ('a_sigma_levels', o3_prof_vmr, {'HSP_atmosphere': 'O3_VMR'}),
        'a_temperature': ('a_sigma_levels', ds['T'].values, {'HSP_atmosphere': 'TEMPERATURE_K'})
    }
    hsp_atmo = xr.Dataset(data_vars=data_vars, attrs={'atmosphere': 'Averaged GEOS-CHEM hybrid sigma pressure NO2 and temperature profiles with pressure and air density profiles estimated assuming ideal gas'})

    return hsp_atmo


def hot_hsp_atmosphere():
    """
    Creates the default HSP (Hybrid Sigma Pressure) atmosphere definition that was used by the first iterations of
    the AMF software.

    Consists of NO2, O3, and temperature profiles at each of the 72 layer centres.

    The AMF code will take these inputs as well as a surface pressure and convert them to altitude-dependent
    profiles for sasktran. The conversion will use the barometric formula assuming ideal gas and lapse rate = 0
    (constant temperature within each layer). It will also produce an air density profiles using the ideal gas law.
    """
    # hard coded settings
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    # take a ozone altitude profile and convert it to HSP assuming surface pressure of 101.325 kPa (atmospheric
    # sea level) at an altitude of 0
    ds = load_ds()

    temp = 1.1 * ds['T'].values
    atmo = HSP_to_altitude_state(101325.0, 0.0, temp)

    altitude = atmo.altitude.values
    density = atmo.density.values
    o3_clim = sk.Labow()
    o3_prof_cm3 = o3_clim.get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)
    o3_prof_vmr = o3_prof_cm3 / density

    # identify variables defining the HSP atmosphere by the attribute 'HSP_atmosphere'
    data_vars = {
        'a_no2_vmr': ('a_sigma_levels', ds['TRCNO2'].values, {'HSP_atmosphere': 'NO2_VMR'}),
        'a_o3_vmr': ('a_sigma_levels', o3_prof_vmr, {'HSP_atmosphere': 'O3_VMR'}),
        'a_temperature': ('a_sigma_levels', temp, {'HSP_atmosphere': 'TEMPERATURE_K'})
    }
    hsp_atmo = xr.Dataset(data_vars=data_vars, attrs={'atmosphere': 'default with temperature profile scaled up by 10%'})

    return hsp_atmo


def hot_trop_hsp_atmosphere():
    """
    Creates the default HSP (Hybrid Sigma Pressure) atmosphere definition that was used by the first iterations of
    the AMF software.

    Consists of NO2, O3, and temperature profiles at each of the 72 layer centres.

    The AMF code will take these inputs as well as a surface pressure and convert them to altitude-dependent
    profiles for sasktran. The conversion will use the barometric formula assuming ideal gas and lapse rate = 0
    (constant temperature within each layer). It will also produce an air density profiles using the ideal gas law.
    """
    # hard coded settings
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    # take a ozone altitude profile and convert it to HSP assuming surface pressure of 101.325 kPa (atmospheric
    # sea level) at an altitude of 0
    ds = load_ds()

    temp = ds['T'].values
    temp[:30] *= 1.1
    atmo = HSP_to_altitude_state(101325.0, 0.0, temp)

    altitude = atmo.altitude.values
    density = atmo.density.values
    o3_clim = sk.Labow()
    o3_prof_cm3 = o3_clim.get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)
    o3_prof_vmr = o3_prof_cm3 / density

    # identify variables defining the HSP atmosphere by the attribute 'HSP_atmosphere'
    data_vars = {
        'a_no2_vmr': ('a_sigma_levels', ds['TRCNO2'].values, {'HSP_atmosphere': 'NO2_VMR'}),
        'a_o3_vmr': ('a_sigma_levels', o3_prof_vmr, {'HSP_atmosphere': 'O3_VMR'}),
        'a_temperature': ('a_sigma_levels', temp, {'HSP_atmosphere': 'TEMPERATURE_K'})
    }
    hsp_atmo = xr.Dataset(data_vars=data_vars, attrs={'atmosphere': 'default with troposphere (first 30 layers) temperature increased by 10%'})

    return hsp_atmo


def hot_boundary_hsp_atmosphere():
    """
    Creates the default HSP (Hybrid Sigma Pressure) atmosphere definition that was used by the first iterations of
    the AMF software.

    Consists of NO2, O3, and temperature profiles at each of the 72 layer centres.

    The AMF code will take these inputs as well as a surface pressure and convert them to altitude-dependent
    profiles for sasktran. The conversion will use the barometric formula assuming ideal gas and lapse rate = 0
    (constant temperature within each layer). It will also produce an air density profiles using the ideal gas law.
    """
    # hard coded settings
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    # take a ozone altitude profile and convert it to HSP assuming surface pressure of 101.325 kPa (atmospheric
    # sea level) at an altitude of 0
    ds = load_ds()

    temp = ds['T'].values
    temp[:5] *= 1.1
    atmo = HSP_to_altitude_state(101325.0, 0.0, temp)

    altitude = atmo.altitude.values
    density = atmo.density.values
    o3_clim = sk.Labow()
    o3_prof_cm3 = o3_clim.get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)
    o3_prof_vmr = o3_prof_cm3 / density

    # identify variables defining the HSP atmosphere by the attribute 'HSP_atmosphere'
    data_vars = {
        'a_no2_vmr': ('a_sigma_levels', ds['TRCNO2'].values, {'HSP_atmosphere': 'NO2_VMR'}),
        'a_o3_vmr': ('a_sigma_levels', o3_prof_vmr, {'HSP_atmosphere': 'O3_VMR'}),
        'a_temperature': ('a_sigma_levels', temp, {'HSP_atmosphere': 'TEMPERATURE_K'})
    }
    hsp_atmo = xr.Dataset(data_vars=data_vars, attrs={'atmosphere': 'default with boundary layer (first 5 layers) temperature increased by 10%'})

    return hsp_atmo


def winter_hsp_atmosphere():
    # hard coded settings
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    # take a ozone altitude profile and convert it to HSP assuming surface pressure of 101.325 kPa (atmospheric
    # sea level) at an altitude of 0
    ds = load_ds(avg=False).isel(time=350)

    atmo = HSP_to_altitude_state(101325.0, 0.0, ds['T'])

    altitude = atmo.altitude.values
    density = atmo.density.values
    o3_clim = sk.Labow()
    o3_prof_cm3 = o3_clim.get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)
    o3_prof_vmr = o3_prof_cm3 / density

    # identify variables defining the HSP atmosphere by the attribute 'HSP_atmosphere'
    data_vars = {
        'a_no2_vmr': ('a_sigma_levels', ds['TRCNO2'].values, {'HSP_atmosphere': 'NO2_VMR'}),
        'a_o3_vmr': ('a_sigma_levels', o3_prof_vmr, {'HSP_atmosphere': 'O3_VMR'}),
        'a_temperature': ('a_sigma_levels', ds['T'].values, {'HSP_atmosphere': 'TEMPERATURE_K'})
    }
    hsp_atmo = xr.Dataset(data_vars=data_vars, attrs={'atmosphere': 'default'})

    return hsp_atmo


def summer_hsp_atmosphere():
    # hard coded settings
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    # take a ozone altitude profile and convert it to HSP assuming surface pressure of 101.325 kPa (atmospheric
    # sea level) at an altitude of 0
    ds = load_ds(avg=False).isel(time=29)

    atmo = HSP_to_altitude_state(101325.0, 0.0, ds['T'])

    altitude = atmo.altitude.values
    density = atmo.density.values
    o3_clim = sk.Labow()
    o3_prof_cm3 = o3_clim.get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)
    o3_prof_vmr = o3_prof_cm3 / density

    # identify variables defining the HSP atmosphere by the attribute 'HSP_atmosphere'
    data_vars = {
        'a_no2_vmr': ('a_sigma_levels', ds['TRCNO2'].values, {'HSP_atmosphere': 'NO2_VMR'}),
        'a_o3_vmr': ('a_sigma_levels', o3_prof_vmr, {'HSP_atmosphere': 'O3_VMR'}),
        'a_temperature': ('a_sigma_levels', ds['T'].values, {'HSP_atmosphere': 'TEMPERATURE_K'})
    }
    hsp_atmo = xr.Dataset(data_vars=data_vars, attrs={'atmosphere': 'default'})

    return hsp_atmo


def ozone_hsp_atmosphere():
    atmo_hsp = default_hsp_atmosphere()
    atmo = hsp_to_altitude(atmo_hsp, 101325, 0)
    alts = _pad(atmo.altitude, atmo.altitude_edges[0], atmo.altitude_edges[-1])
    o3 = _pad(atmo.o3)
    from scipy.integrate import simps
    column = simps(o3, 1e2 * alts)
    column /= 2.687e16  # convert to DU
    profile = atmo_hsp.a_o3_vmr
    atmo_hsp['a_o3_vmr'] = profile * 350.0 / column
    return atmo_hsp


def no2_hsp_atmosphere():
    atmo = default_hsp_atmosphere()
    atmo['a_no2_vmr'] *= 1.5
    return atmo


def no2_boundary_hsp_atmosphere():
    atmo = default_hsp_atmosphere()
    atmo.a_no2_vmr.plot(y='a_sigma_levels')
    boundary = atmo['a_no2_vmr'][:15]
    boundary *= 100
    return atmo


def no2_no_boundary_hsp_atmosphere():
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    atmo_hsp = default_hsp_atmosphere()
    atmo_alt = hsp_to_altitude(atmo_hsp)
    alts = atmo_alt.altitude
    no2_cm3 = sk.Pratmo().get_parameter('SKCLIMATOLOGY_NO2_CM3', latitude, longitude, alts, time)
    no2_vmr = no2_cm3 / sk.MSIS90().get_parameter('SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3', latitude, longitude, alts, time)
    atmo_hsp['a_no2_vmr'] = ('a_sigma_levels', no2_vmr, atmo_hsp.a_no2_vmr.attrs)
    return atmo_hsp


def is_hsp_atmosphere(hsp_atmo):
    # fetch HSP profiles from dataset
    no2_vmr, o3_vmr, temperature_k = None, None, None
    for var in hsp_atmo.variables:
        profile = hsp_atmo[var].attrs.get('HSP_atmosphere')
        if profile == 'NO2_VMR':
            no2_vmr = hsp_atmo[var].values
        elif profile == 'O3_VMR':
            o3_vmr = hsp_atmo[var].values
        elif profile == 'TEMPERATURE_K':
            temperature_k = hsp_atmo[var].values

    if no2_vmr is None or temperature_k is None:
        return False
    else:
        return True


def default_static_atmosphere():
    # hard coded settings
    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    altitude = np.arange(0, 80001, 500)
    pressure = sk.MSIS90().get_parameter('SKCLIMATOLOGY_PRESSURE_PA', latitude, longitude, altitude, time)
    temperature = sk.MSIS90().get_parameter('SKCLIMATOLOGY_TEMPERATURE_K', latitude, longitude, altitude, time)
    air = sk.MSIS90().get_parameter('SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3', latitude, longitude, altitude, time)
    no2 = sk.Pratmo().get_parameter('SKCLIMATOLOGY_NO2_CM3', latitude, longitude, altitude, time)
    o3 = sk.Labow().get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)

    coords = {
        'a_altitude_m': ('a_altitude', altitude, {'static_atmosphere': 'ALTITUDE_M'})
    }
    data_vars = {
        'a_pressure_pa': ('a_altitude', pressure, {'static_atmosphere': 'PRESSURE_PA'}),
        'a_temperature_k': ('a_altitude', temperature, {'static_atmosphere': 'TEMPERATURE_K'}),
        'a_density_cm3': ('a_altitude', air, {'static_atmosphere': 'AIRNUMBERDENSITY_CM3'}),
        'a_no2_cm3': ('a_altitude', no2, {'static_atmosphere': 'NO2_CM3'}),
        'a_o3_cm3': ('a_altitude', o3, {'static_atmosphere': 'O3_CM3'})
    }
    return xr.Dataset(data_vars=data_vars, coords=coords, attrs={'atmosphere': 'default static atmosphere: MSIS90, Pratmo and Labow at 57.1N 248.4E MJD58197.666'})


def static_hsp_atmosphere():
    ds = load_ds()
    atmo = HSP_to_altitude_state(1e5, 0.0, ds['T'])

    kind = 'quadratic'
    density = interp1d(atmo.altitude.values, atmo.density.values, kind=kind, fill_value='extrapolate')
    pressure = interp1d(atmo.altitude.values, atmo.pressure.values, kind=kind, fill_value='extrapolate')
    temperature = interp1d(atmo.altitude.values, atmo.temperature.values, kind=kind, fill_value='extrapolate')
    no2 = interp1d(atmo.altitude.values, ds['TRCNO2'], kind=kind, fill_value='extrapolate')

    latitude = 57.1
    longitude = 248.4
    time = 58197.666

    altitude = np.arange(0, 75001, 500)
    pressure = pressure(altitude)
    temperature = temperature(altitude)
    air = density(altitude)
    no2 = no2(altitude)
    o3 = sk.Labow().get_parameter('SKCLIMATOLOGY_O3_CM3', latitude, longitude, altitude, time)

    coords = {
        'a_altitude_m': ('a_altitude', altitude, {'static_atmosphere': 'ALTITUDE_M'})
    }
    data_vars = {
        'a_pressure_pa': ('a_altitude', pressure, {'static_atmosphere': 'PRESSURE_PA'}),
        'a_temperature_k': ('a_altitude', temperature, {'static_atmosphere': 'TEMPERATURE_K'}),
        'a_density_cm3': ('a_altitude', air, {'static_atmosphere': 'AIRNUMBERDENSITY_CM3'}),
        'a_no2_cm3': ('a_altitude', no2 * air, {'static_atmosphere': 'NO2_CM3'}),
        'a_o3_cm3': ('a_altitude', o3, {'static_atmosphere': 'O3_CM3'})
    }

    return xr.Dataset(data_vars=data_vars, coords=coords, attrs={'atmosphere': 'static atmosphere that matches the default HSP atmosphere assuming a surface pressure of 100 kPa'})


def hot_static_atmosphere():
    atmo = default_static_atmosphere()
    atmo['a_temperature_k'] *= 1.1
    atmo.attrs['atmosphere'] += ', temperature increased by 10%'
    return atmo


def dense_static_atmosphere():
    atmo = default_static_atmosphere()
    atmo['a_density_cm3'] *= 1.1
    atmo['a_pressure_pa'] *= 1.1
    atmo.attrs['atmosphere'] += ', air density and pressure increased by 10%'
    return atmo


def scaled_static_atmosphere(m=0.0, b=1.0):
    atmo = default_static_atmosphere()
    atmo['a_pressure_pa'] *= (m * atmo['a_altitude_m'].values + b)
    atmo['a_density_cm3'] *= (m * atmo['a_altitude_m'].values + b)
    atmo.attrs['atmosphere'] += f', air density and pressure scaled by ({m} m^-1) x altitude + ({b})'
    return atmo


def short_static():
    return default_static_atmosphere().isel(a_altitude=range(33))


def is_static_atmosphere(static_atmo):
    # fetch HSP profiles from dataset
    altitude_m, no2_cm3, o3_cm3, temperature_k, air_cm3, pressure_pa = None, None, None, None, None, None
    for var in static_atmo.variables:
        profile = static_atmo[var].attrs.get('static_atmosphere')
        if profile == 'ALTITUDE_M':
            altitude_m = static_atmo[var].values
        elif profile == 'NO2_CM3':
            no2_cm3 = static_atmo[var].values
        elif profile == 'O3_CM3':
            o3_cm3 = static_atmo[var].values
        elif profile == 'TEMPERATURE_K':
            temperature_k = static_atmo[var].values
        elif profile == 'AIRNUMBERDENSITY_CM3':
            air_cm3 = static_atmo[var].values
        elif profile == 'PRESSURE_PA':
            pressure_pa = static_atmo[var].values

    if altitude_m is None or no2_cm3 is None or temperature_k is None or air_cm3 is None or pressure_pa is None:
        return False
    else:
        return True


def hsp_to_altitude(hsp_atmo: xr.Dataset, p_surface_Pa: float=101325.0, h_surface: float=0.0):
    # fetch HSP profiles from dataset
    no2_vmr, o3_vmr, temperature_k = None, None, None
    for var in hsp_atmo.variables:
        profile = hsp_atmo[var].attrs.get('HSP_atmosphere')
        if profile == 'NO2_VMR':
            no2_vmr = hsp_atmo[var].values
        elif profile == 'O3_VMR':
            o3_vmr = hsp_atmo[var].values
        elif profile == 'TEMPERATURE_K':
            temperature_k = hsp_atmo[var].values

    if no2_vmr is None:
        raise ValueError('HSP NO2 profile not found in dataset')

    if temperature_k is None:
        raise ValueError('HSP temperature profile not found in dataset')

    # calculate pressure, temperature, and density altitude profiles
    state = HSP_to_altitude_state(p_surface_Pa, h_surface, temperature_k)

    state['no2'] = ('altitude', no2_vmr * state.density.values)
    if o3_vmr is not None:
        state['o3'] = ('altitude', o3_vmr * state.density.values)

    return state


def hsp_to_sasktran(hsp_atmo: xr.Dataset, p_surface_Pa: float, h_surface: float):
    alt_atmo = hsp_to_altitude(hsp_atmo, p_surface_Pa, h_surface)
    # construct sasktran atmosphere object
    altitude = np.array([alt_atmo.surface_altitude.item(),
                         *alt_atmo.altitude,
                         alt_atmo.top_of_atmosphere_altitude.item()])
    pressure = _pad(alt_atmo.pressure.values, last=0)
    temperature = _pad(alt_atmo.temperature.values)
    air = _pad(alt_atmo.density.values, last=0)
    no2 = _pad(alt_atmo.no2.values, last=0)
    if 'o3' in alt_atmo.variables:
        o3 = _pad(alt_atmo.o3.values, last=0)

    atmo = sk.Atmosphere()

    atmo.atmospheric_state = sk.ClimatologyUserDefined(altitude, {
        'SKCLIMATOLOGY_PRESSURE_PA': pressure,
        'SKCLIMATOLOGY_TEMPERATURE_K': temperature
    })
    atmo['air'] = sk.Species(sk.Rayleigh(), sk.ClimatologyUserDefined(altitude, {'SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3': air}))
    atmo['no2'] = sk.Species(sk.NO2Vandaele1998(), sk.ClimatologyUserDefined(altitude, {'SKCLIMATOLOGY_NO2_CM3': no2}))
    if 'o3' in alt_atmo.variables:
        atmo['o3'] = sk.Species(sk.O3DBM(), sk.ClimatologyUserDefined(altitude, {'SKCLIMATOLOGY_O3_CM3': o3}))

    info = {
        'surface_altitude': alt_atmo.surface_altitude.item(),
        'top_of_atmosphere_altitude': alt_atmo.top_of_atmosphere_altitude.item(),
        'altitude': alt_atmo.altitude.values,
        'altitude_edges': alt_atmo.altitude_edges.values,
        'pressure_edges': alt_atmo.pressure_edges.values
    }

    return atmo, info


def static_to_sasktran(static_atmo: xr.Dataset, p_surface_Pa: float, h_surface: float):
    # retrieve the profiles stored in the table
    altitude_m, no2_cm3, o3_cm3, temperature_k, air_cm3, pressure_pa = None, None, None, None, None, None
    for var in static_atmo.variables:
        profile = static_atmo[var].attrs.get('static_atmosphere')
        if profile == 'ALTITUDE_M':
            altitude_m = static_atmo[var].values
        elif profile == 'NO2_CM3':
            no2_cm3 = static_atmo[var].values
        elif profile == 'O3_CM3':
            o3_cm3 = static_atmo[var].values
        elif profile == 'TEMPERATURE_K':
            temperature_k = static_atmo[var].values
        elif profile == 'AIRNUMBERDENSITY_CM3':
            air_cm3 = static_atmo[var].values
        elif profile == 'PRESSURE_PA':
            pressure_pa = static_atmo[var].values

    # construct sasktran atmosphere object

    # get the altitude matching the surface pressure
    ground = interp1d(np.log(pressure_pa[::-1]), altitude_m[::-1], fill_value='extrapolate')(np.log(p_surface_Pa))

    # use an altitude grid shifted up to the ground altitude (but clipped at the same maximum value)
    altitude = altitude_m + ground
    altitude = altitude[altitude < np.max(altitude_m)]

    # interpolate the profiles to the new altitude grid
    if no2_cm3 is not None:
        no2_cm3 = interp1d(altitude_m, no2_cm3, fill_value='extrapolate')(altitude)
    if o3_cm3 is not None:
        o3_cm3 = interp1d(altitude_m, o3_cm3, fill_value='extrapolate')(altitude)
    if temperature_k is not None:
        temperature_k = interp1d(altitude_m, temperature_k, fill_value='extrapolate')(altitude)
    if pressure_pa is not None:
        pressure_pa = interp1d(altitude_m, pressure_pa, fill_value='extrapolate')(altitude)
    if air_cm3 is not None:
        air_cm3 = interp1d(altitude_m, air_cm3, fill_value='extrapolate')(altitude)

    altitude += h_surface - ground

    atmo = sk.Atmosphere()
    atmo.atmospheric_state = sk.ClimatologyUserDefined(altitude, {
        'SKCLIMATOLOGY_PRESSURE_PA': pressure_pa,
        'SKCLIMATOLOGY_TEMPERATURE_K': temperature_k
    })
    atmo['air'] = sk.Species(sk.Rayleigh(), sk.ClimatologyUserDefined(altitude, {'SKCLIMATOLOGY_AIRNUMBERDENSITY_CM3': air_cm3}))
    atmo['no2'] = sk.Species(sk.NO2Vandaele1998(), sk.ClimatologyUserDefined(altitude, {'SKCLIMATOLOGY_NO2_CM3': no2_cm3}))
    if o3_cm3 is not None:
        atmo['o3'] = sk.Species(sk.O3DBM(), sk.ClimatologyUserDefined(altitude, {'SKCLIMATOLOGY_O3_CM3': o3_cm3}))

    info = {
        'surface_altitude': np.min(altitude),
        'top_of_atmosphere_altitude': np.max(altitude),
        'altitude': altitude,
        'altitude_edges': altitude,
        'pressure_edges': pressure_pa
    }

    return atmo, info


def _pad(array, first=None, last=None):
    """ Add a value to the beginning and to the end of array. None (default) adds a duplicate of the first and last elements. """
    first = array[0] if first is None else first
    last = array[-1] if last is None else last
    return np.array([first, *array, last])


if __name__ == '__main__':
    import os
    import skamf
    atmo_path = os.path.join(os.path.dirname(os.path.dirname(skamf.__file__)), 'atmosphere')

    default = default_hsp_atmosphere()
    default.to_netcdf(os.path.join(atmo_path, 'default.nc'))
    hot = hot_hsp_atmosphere()
    hot.to_netcdf(os.path.join(atmo_path, 'hot.nc'))
    trop = hot_trop_hsp_atmosphere()
    trop.to_netcdf(os.path.join(atmo_path, 'hot_trop.nc'))
    bound = hot_boundary_hsp_atmosphere()
    bound.to_netcdf(os.path.join(atmo_path, 'hot_boundary.nc'))
    winter = winter_hsp_atmosphere()
    winter.to_netcdf(os.path.join(atmo_path, 'winter.nc'))
    summer = summer_hsp_atmosphere()
    summer.to_netcdf(os.path.join(atmo_path, 'summer.nc'))
    static = default_static_atmosphere()
    static.to_netcdf(os.path.join(atmo_path, 'static.nc'))
    static_hsp = static_hsp_atmosphere()
    static_hsp.to_netcdf(os.path.join(atmo_path, 'static_hsp.nc'))
    hot_static = hot_static_atmosphere()
    hot_static.to_netcdf(os.path.join(atmo_path, 'hot_static.nc'))
    m, b = 2.5e-6, 10
    scaled_static = scaled_static_atmosphere(m, b)
    scaled_static.to_netcdf(os.path.join(atmo_path, f'scaled_static_{"" if m > 0 else "m"}{int(m*1e8)}_{int(b*1e2)}.nc'))
    dense = dense_static_atmosphere()
    dense.to_netcdf(os.path.join(atmo_path, 'dense.nc'))
    short = short_static()
    short.to_netcdf(os.path.join(atmo_path, 'short.nc'))
    ozone = ozone_hsp_atmosphere()
    ozone.to_netcdf(os.path.join(atmo_path, 'ozone.nc'))
    no2 = no2_hsp_atmosphere()
    no2.to_netcdf(os.path.join(atmo_path, 'no2.nc'))
    no2_boundary = no2_boundary_hsp_atmosphere()
    no2_boundary.to_netcdf(os.path.join(atmo_path, 'no2_boundary.nc'))
    no2_no_boundary = no2_no_boundary_hsp_atmosphere()
    no2_no_boundary.to_netcdf(os.path.join(atmo_path, 'no2_no_boundary.nc'))
