"""
This file generates a new AMF table.
"""

import numpy as np
import xarray as xr

from skamf.utility import get_response
from skamf.atmosphere import is_hsp_atmosphere, default_hsp_atmosphere


def new_table(clear_sky: bool, cloudy_sky: bool, total_sum: bool, total_remove: bool, engine: str,
              **kwargs) -> xr.Dataset:
    print('Welcome to the AMF table wizard.')
    print('You will be prompted for the new AMF tables coordinates.')
    print('Coordinate values should be delimited by spaces or commas.')
    print("Respond with 'omit' to not include a coordinate in the table.")
    print('An empty response will select the default values.')
    print("Enter 'exit' to quit without making the table.")
    print('\n' + '-' * 80)

    table = xr.Dataset()

    clear_coords = []
    cloudy_coords = []

    # Get box altitudes
    values = get_response(
        'box_altitude',
        "Box altitudes [m] (default: '250, 750, ..., 15725' OR 'linspace, 250, 15725, 32' OR 'arange, 250, 16000, 500')",
        np.arange(250, 16000, 500).astype(float),
        True
    )
    if values is not None:
        clear_coords.append('box_altitude')
        cloudy_coords.append('box_altitude')
        table.coords['box_altitude'] = values
        table.coords['box_altitude'].attrs['units'] = 'm'
        table.coords['box_altitude'].attrs['positive'] = 'up'
        table.coords['box_altitude'].attrs['long_name'] = 'box altitude'
        table.coords['box_altitude'].attrs['standard_name'] = 'altitude'
        print(table.coords['box_altitude'])
    print('\n' + '-' * 80)

    if clear_sky:
        # Get surface albedo
        values = get_response(
            'surface_albedo',
            "Surface albedo [unitless] (default: '0, 0.03, 0.06, 0.09, 0.12, 0.20, 0.30, 0.50, 0.75, 1.00')",
            np.array([0, 0.03, 0.06, 0.09, 0.12, 0.20, 0.30, 0.50, 0.75, 1.00])
        )
        if values is not None:
            clear_coords.append('surface_albedo')
            table.coords['surface_albedo'] = values
            table.coords['surface_albedo'].attrs['units'] = '1'
            table.coords['surface_albedo'].attrs['long_name'] = 'surface albedo'
            table.coords['surface_albedo'].attrs['standard_name'] = 'surface_albedo'
            print(table.coords['surface_albedo'])
        print('\n' + '-' * 80)

    # Get surface pressure
    values = get_response(
        'surface_pressure',
        "Surface pressure [kPa] (default: '60, 80, 90, 100')",
        np.array([60, 80, 90, 100]).astype(float)
    )
    if values is not None:
        clear_coords.append('surface_pressure')
        cloudy_coords.append('surface_pressure')
        table.coords['surface_pressure'] = 1e3 * values
        table.coords['surface_pressure'].attrs['units'] = 'Pa'
        table.coords['surface_pressure'].attrs['long_name'] = 'surface air pressure'
        table.coords['surface_pressure'].attrs['standard_name'] = 'surface_air_pressure'
        print(table.coords['surface_pressure'])
    print('\n' + '-' * 80)

    if cloudy_sky:
        # Get cloud top pressure
        values = get_response(
            'cloud_top_pressure',
            "Pressure at cloud top [kPa] (default: '20, 40, 60, 80, 90')",
            np.array([20, 40, 60, 80, 90]).astype(float)
        )
        if values is not None:
            cloudy_coords.append('cloud_top_pressure')
            table.coords['cloud_top_pressure'] = 1e3 * values
            table.coords['cloud_top_pressure'].attrs['units'] = 'Pa'
            table.coords['cloud_top_pressure'].attrs['long_name'] = 'air pressure at cloud top'
            table.coords['cloud_top_pressure'].attrs['standard_name'] = 'air_pressure_at_cloud_top'
            print(table.coords['cloud_top_pressure'])
        print('\n' + '-' * 80)

        values = get_response(
            'cloud_albedo',
            "Cloud albedo [unitless] (default: '0.8')",
            np.array([0.8])
        )
        if values is not None:
            cloudy_coords.append('cloud_albedo')
            table.coords['cloud_albedo'] = values
            table.coords['cloud_albedo'].attrs['units'] = '1'
            table.coords['cloud_albedo'].attrs['long_name'] = 'albedo of cloud'
            table.coords['cloud_albedo'].attrs['standard_name'] = 'cloud_albedo'
            print(table.coords['cloud_albedo'])
        print('\n' + '-' * 80)

    # Get solar zenith
    values = get_response(
        'solar_zenith',
        "Solar zenith angle [deg] (default: '0, 30, 50, 60, 65, 70, 73, 76, 78, 80')",
        np.array([0, 30, 50, 60, 65, 70, 73, 76, 78, 80]).astype(float)
    )
    if values is not None:
        clear_coords.append('solar_zenith')
        cloudy_coords.append('solar_zenith')
        table.coords['solar_zenith'] = values
        table.coords['solar_zenith'].attrs['units'] = 'degree'
        table.coords['solar_zenith'].attrs['long_name'] = 'solar zenith angle'
        table.coords['solar_zenith'].attrs['standard_name'] = 'solar_zenith_angle'
        print(table.coords['solar_zenith'])
    print('\n' + '-' * 80)

    # Get viewing zenith
    values = get_response(
        'viewing_zenith',
        "Viewing zenith angle [deg] (default: '0, 30, 50, 60, 65, 70, 72')",
        np.array([0, 30, 50, 60, 65, 70, 72]).astype(float)
    )
    if values is not None:
        clear_coords.append('viewing_zenith')
        cloudy_coords.append('viewing_zenith')
        table.coords['viewing_zenith'] = values
        table.coords['viewing_zenith'].attrs['units'] = 'degree'
        table.coords['viewing_zenith'].attrs['long_name'] = 'line of sight zenith angle'
        table.coords['viewing_zenith'].attrs['standard_name'] = 'platform_zenith_angle'
        print(table.coords['viewing_zenith'])
    print('\n' + '-' * 80)

    # Get azimuth differences
    values = get_response(
        'azimuth_difference',
        "Difference between viewing and solar azimuth angles [deg] (default: '0, 30, ..., 180')",
        np.arange(0, 181, 30).astype(float)
    )
    if values is not None:
        clear_coords.append('azimuth_difference')
        cloudy_coords.append('azimuth_difference')
        table.coords['azimuth_difference'] = values
        table.coords['azimuth_difference'].attrs['units'] = 'degree'
        table.coords['azimuth_difference'].attrs['long_name'] = 'azimuth angle difference'
        table.coords['azimuth_difference'].attrs['standard_name'] = 'angle_of_rotation_from_solar_azimuth_to_platform_azimuth'
        print(table.coords['azimuth_difference'])
    print('\n' + '-' * 80)

    if clear_sky:
        table['clear_box_amf'] = xr.DataArray(
            data=np.zeros([table[c].shape[0] for c in clear_coords]),
            coords=[table[c] for c in clear_coords]
        )
        table['clear_box_amf'].attrs['long_name'] = 'NO2 box air mass factors'
        table['clear_box_amf'].attrs['units'] = '1'

        if engine == 'mc':
            table['clear_box_amf_std'] = xr.DataArray(
                data=np.zeros([table[c].shape[0] for c in clear_coords]),
                coords=[table[c] for c in clear_coords]
            )
            table['clear_box_amf_std'].attrs['long_name'] = 'NO2 box air mass factor standard deviations'
            table['clear_box_amf_std'].attrs['units'] = '1'

        clear_coords.remove('box_altitude')
        if total_sum:
            table['clear_amf_box_sum'] = xr.DataArray(
                data=np.zeros([table[c].shape[0] for c in clear_coords]),
                coords=[table[c] for c in clear_coords]
            )
        if total_remove:
            table['clear_amf_no_NO2'] = xr.DataArray(
                data=np.zeros([table[c].shape[0] for c in clear_coords]),
                coords=[table[c] for c in clear_coords]
            )
        del clear_coords
    if cloudy_sky:
        table['cloudy_box_amf'] = xr.DataArray(
            data=np.zeros([table[c].shape[0] for c in cloudy_coords]),
            coords=[table[c] for c in cloudy_coords]
        )
        table['cloudy_box_amf'].attrs['long_name'] = 'NO2 box air mass factors'
        table['cloudy_box_amf'].attrs['units'] = '1'

        if engine == 'mc':
            table['cloudy_box_amf_std'] = xr.DataArray(
                data=np.zeros([table[c].shape[0] for c in cloudy_coords]),
                coords=[table[c] for c in cloudy_coords]
            )
            table['cloudy_box_amf_std'].attrs['long_name'] = 'NO2 box air mass factor standard deviations'
            table['cloudy_box_amf_std'].attrs['units'] = '1'

        cloudy_coords.remove('box_altitude')
        if total_sum:
            table['cloudy_amf_box_sum'] = xr.DataArray(
                data=np.zeros([table[c].shape[0] for c in cloudy_coords]),
                coords=[table[c] for c in cloudy_coords]
            )
        if total_remove:
            table['cloudy_amf_no_NO2'] = xr.DataArray(
                data=np.zeros([table[c].shape[0] for c in cloudy_coords]),
                coords=[table[c] for c in cloudy_coords]
            )
        del cloudy_coords
    return table


def create_table(filename, copy=None, copy_options=None, clear_sky=False, cloudy_sky=False, atmosphere=None, total_sum=False, total_remove=False, engine=None):
    # create the base table with no options, either with the new table dialog or by copying an existing table
    # if an existing table is copied, keep the atmosphere vars for now (will be overwritten if an atmosphere is specified)
    if copy is None:
        fresh_table = new_table(clear_sky, cloudy_sky, total_sum, total_remove, engine)
    else:
        with xr.open_dataset(copy).load() as old_table:
            fresh_table = old_table.copy()
        # if we are copying an MC table with many trials, we just want to copy 1 of them
        if 'trials' in fresh_table.dims:
            fresh_table = fresh_table.sel(trials=0)
            fresh_table = fresh_table.drop(['date_created', 'version'])
        for key in fresh_table.data_vars:
            if all([a not in fresh_table[key].attrs for a in ['HSP_atmosphere', 'static_atmosphere']]):
                fresh_table[key].values = np.zeros(fresh_table[key].shape)
            if 'version' in fresh_table[key].attrs:
                del fresh_table[key].attrs['version']
        # remove options
        options = []
        for key in fresh_table.data_vars:
            if 'sasktran_option' in fresh_table[key].attrs:
                options.append(key)
        fresh_table = fresh_table.drop(options)
        # remove attributes associated with a table in progress or completed
        for key in ['date_created', 'progress_file', 'radiative_transfer_engine', 'version']:
            if key in fresh_table.attrs:
                del fresh_table.attrs[key]

    # copy options from another table if specified
    # set the permitted engine attribute if specified.
    if copy_options is not None:
        with xr.open_dataset(copy_options).load() as options_table:
            keys = list(fresh_table.variables.keys())
            for key in keys:
                if 'sasktran_option' in fresh_table[key].attrs:
                    del fresh_table[key]
            for key in options_table.data_vars:
                if 'sasktran_option' in options_table[key].attrs:
                    fresh_table[key] = options_table[key].drop(list(options_table[key].coords.keys()))
            for key in options_table.coords:
                if 'sasktran_option' in options_table[key].attrs:
                    fresh_table.coords[key] = options_table[key].drop(list(options_table[key].coords.keys()))
            if 'permitted_engine' in options_table.attrs:
                fresh_table.attrs['permitted_engine'] = options_table.attrs['permitted_engine']
            options_table.close()
    else:
        fresh_table.attrs['permitted_engine'] = engine

    # replace existing atmosphere variables (if any) with specified atmosphere
    merge = True
    if atmosphere is not None:
        with xr.open_dataset(atmosphere) as atmo:
            is_hsp_atmosphere(atmo)
            if 'atmosphere' in fresh_table.attrs:
                del fresh_table.attrs['atmosphere']
                remove = [var for var in fresh_table.variables if 'HSP_atmosphere' in fresh_table[var].attrs]
                remove += [var for var in fresh_table.variables if 'static_atmosphere' in fresh_table[var].attrs]
                for var in remove:
                    del fresh_table[var]
    else:
        if 'atmosphere' not in fresh_table.attrs:
            print('no atmosphere was specified so the default hsp atmosphere was used')
            atmo = default_hsp_atmosphere()
        else:
            atmo = None
            merge = False
    if merge:
        attrs = atmo.attrs
        attrs.update(fresh_table.attrs)
        fresh_table = xr.merge([fresh_table, atmo])
        fresh_table.attrs = attrs

    fresh_table.to_netcdf(filename)
    return fresh_table


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser(description="Generate an new empty AMF table.")
    parser.add_argument('filename', type=str, help='path to the location the table will be saved to.')
    parser.add_argument('-c', '--copy', type=str, help='table to copy coords and data_vars from')
    parser.add_argument('--copy-options', type=str, help='table to copy options (including permitted engines) from')
    parser.add_argument('--clear-sky', action='store_true')
    parser.add_argument('--cloudy-sky', action='store_true')
    parser.add_argument('-a', '--atmosphere', type=str, help='path to HSP atmosphere')
    parser.add_argument('--total-sum', action='store_true', help='set to calculate the total AMF by summing the box-AMFs')
    parser.add_argument('--total-remove', action='store_true', help='set to estimate the total AMF by comparing radiances with the absorber removed')
    parser.add_argument('-e', '--engine', type=str, choices=['hr', 'mc', 'do', 'disco'], default='do', help='engine that will be allowed to process the table - defaults to do')

    args = vars(parser.parse_args())

    if args['copy'] is None and args['clear_sky'] is False and args['cloudy_sky'] is False:
        parser.error("At least one of '--clear-sky' and '--cloudy-sky' must be selected")

    if args['engine'] is None and args['copy_options'] is None:
        parser.error("If '--copy-options' is not selected then '--engine' must be set")

    table = create_table(**args)
    print('Created empty AMF table:')
    print(table)
