import argparse
import os
import pickle
import logging
import warnings

import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import xarray as xr

import skamf.cf
from skamf.skamflogging import TqdmHandler

warnings.simplefilter(action='ignore', category=FutureWarning)

# setup logging
log = logging.getLogger('AMF Visualizer')
handler = TqdmHandler()
formatter = logging.Formatter(
    '%(asctime)s %(levelname)-8s %(message)s')
handler.setFormatter(formatter)
log.addHandler(handler)
log.setLevel(logging.DEBUG)

parser = argparse.ArgumentParser(description="AMF table visualization tools.")
parser.add_argument('amf_table', type=str, help='path to the AMF table')
parser.add_argument('-t', '--type', type=str, nargs='+', help='plot type followed by dimensions', default=['print'])
parser.add_argument('-v', '--variable', type=str, help='name of the variable')
parser.add_argument('-i', '--coord-index', action='append', nargs=2, help='select a coordinate index', default=[])
parser.add_argument('-n', '--coord-near', action='append', nargs=2, help='select a coordinate near the given value', default=[])
parser.add_argument('-c', '--coord-file', type=str, help='coordinates file')
parser.add_argument('--azimuth', type=str, help='name of the azimuthal coordinate')
parser.add_argument('--zenith', type=str, help='name of the zenith coordinate')
parser.add_argument('--x', type=str, help='label of the x axes (only one of --x or --y can be used)')
parser.add_argument('--y', type=str, help='label of the y axes (only one of --x or --y can be used)')
parser.add_argument('--other', type=str, help='path to the other AMF table that will be compared with respect to')
parser.add_argument('--tol', type=float, help='a tolerance value', default=0.02)
args = vars(parser.parse_args())

if args['coord_file'] is not None:
    with open(args['coord_file'], 'rb') as file:
        loaded_coords = pickle.load(file)
    for c in loaded_coords['coord_index']:
        args['coord_index'].append(c)
    for c in loaded_coords['coord_near']:
        args['coord_near'].append(c)

amf_table = xr.open_dataset(args['amf_table'])

if args['coord_index'] is not None:
    amf_table = amf_table.isel(**{n: int(v) for (n, v) in args['coord_index']})

if args['coord_near'] is not None:
    amf_table = amf_table.sel(**{n: float(v) for (n, v) in args['coord_near']}, method='nearest')

if args['variable'] is not None and args['variable'] != 'all':
    amf_table = amf_table[args['variable']]

log.info(f'Selected table: {amf_table}')

if args['type'][0] == 'coord-file':
    if len(args['type']) != 2:
        parser.error('for type=coord-file the second type argument must be the filename')
    log.info(f'Generating coordinate file: {args["type"][1]}')
    s = {}
    s['coord_index'] = args['coord_index']
    s['coord_near'] = args['coord_near']
    with open(args['type'][1], 'wb') as file:
        pickle.dump(s, file)
    exit(0)

if args['type'][0] == 'polar':
    from scipy.interpolate import griddata
    if isinstance(amf_table, xr.Dataset):
        parser.error('a dataset variable must be selected (-v argument)')
    amf_table = amf_table.to_dataset()
    if args['azimuth'] is None or args['zenith'] is None:
        parser.error('--azimuth and --zenith arguments must be specified')
    azimuths = amf_table.cf[args['azimuth']]
    zeniths = amf_table.cf[args['zenith']]
    amf_table = amf_table.to_array()
    values_coarse = np.array(amf_table.squeeze().transpose(azimuths.name, zeniths.name))

    theta_coarse, r_coarse = np.meshgrid(azimuths, zeniths, indexing='ij')
    points = np.array((theta_coarse.flatten(), r_coarse.flatten())).transpose()

    theta, r = np.meshgrid(
        np.linspace(azimuths[0].item(), azimuths[-1].item(), 1000),
        np.linspace(zeniths[0].item(), zeniths[-1].item(), 1000),
        indexing='ij'
    )

    values = griddata(points, values_coarse.flatten(), (theta, r), method='linear')

    fig = plt.figure()
    ax1 = plt.subplot2grid((5, 10), (0, 0), colspan=9, rowspan=4, projection='polar')
    ax2 = plt.subplot2grid((5, 10), (0, 9), colspan=1, rowspan=5)
    ax3 = plt.subplot2grid((5, 10), (4, 0), colspan=9, rowspan=1)

    contourf = ax1.contourf(theta, r, values)
    plt.colorbar(contourf, ax2)

    ax1.set_thetamin(0)
    ax1.set_thetamax(180)

    plt.suptitle(args['variable'])
    ax3.axis('off')

    coords_to_label = amf_table.squeeze().drop([azimuths.name, zeniths.name, 'variable']).coords
    textbox = ""
    for c in coords_to_label:
        textbox += f'{c}: {amf_table[c].item()} [{amf_table[c].attrs["units"]}]\n'

    textbox = textbox[:-1]

    props = dict(facecolor='white')
    ax3.text(0.1, 0.7, textbox, transform=ax3.transAxes, fontsize=12,
             verticalalignment='center', bbox=props)

    plt.show()

if args['type'][0] == 'plot':
    if isinstance(amf_table, xr.Dataset):
        parser.error('a dataset variable must be selected (-v argument)')
    kwargs = {}
    if args['x'] is not None:
        kwargs['x'] = args['x']
    if args['y'] is not None:
        kwargs['y'] = args['y']
    amf_table.plot(**kwargs)
    plt.show()

if args['type'][0] == 'diff_hist':
    if isinstance(amf_table, xr.Dataset):
        parser.error('a dataset variable must be selected (-v argument)')
    if args['other'] is None:
        parser.error('a dataset to compare to is required (--other argument)')
    coords = amf_table.coords.keys()
    import matplotlib.pyplot as plt

    base_da = xr.open_dataset(args['other'])[args['variable']]
    diff = (amf_table - base_da) / base_da

    fig, axes = plt.subplots(np.ceil(len(coords) / 2).astype(int), 2)

    count = 0

    max_positive_count = 0
    max_negative_count = 0

    def plot(key):
        global max_positive_count, max_negative_count
        sum_across = set(amf_table.coords.keys())
        sum_across.remove(key)
        binned = (diff > args['tol']).astype(int).sum(sum_across)
        max_positive_count = max(max_positive_count, binned.max())
        negative_binned = (diff < -args['tol']).astype(int).sum(sum_across)
        max_negative_count = max(max_negative_count, negative_binned.max())
        plt.sca(axes[count // 2, count % 2])
        plt.bar(np.cumsum(np.ones_like(binned)), binned)
        plt.bar(np.cumsum(np.ones_like(negative_binned)), -negative_binned)
        plt.xticks(np.arange(1, len(binned) + 1, len(binned) // 3), binned[key][::len(binned) // 3].values.astype(str))
        plt.xlabel(f'{key} [{binned[key].attrs["units"]}]')

    for c in coords:
        plot(c)
        count += 1

    for i in range(count):
        plt.sca(axes[i // 2, i % 2])
        plt.ylim([-max_negative_count, max_positive_count])

    plt.suptitle(f'Relative difference in {args["variable"]}\nw.r.t. : {args["other"]}')
    plt.tight_layout(rect=[0, 0.03, 1, 0.95])
    plt.show()
